package be.iminds.ilabt.jfed.experimenter_gui.util;

import javafx.application.HostServices;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.inject.Singleton;
import java.net.URI;
import java.net.URL;

/**
 * User: twalcari
 * Date: 2/4/2015
 * Time: 10:16
 */
@Singleton
public class BrowserUtil {
    private static final Logger LOG = LoggerFactory.getLogger(BrowserUtil.class);

    private final Provider<HostServices> hostServicesProvider;

    @Inject
    public BrowserUtil(Provider<HostServices> hostServicesProvider) {
        this.hostServicesProvider = hostServicesProvider;
    }

    public boolean openUrlInBrowser(URL url){
        return openUrlInBrowser(url.toExternalForm());
    }

    public boolean openUrlInBrowser(URI uri){
        return openUrlInBrowser(uri.toASCIIString());
    }

    public boolean openUrlInBrowser(String url){try {
        hostServicesProvider.get().showDocument(url);
    } catch (Throwable e1) {
        LOG.error("Error while trying to open external browser. Will try to work around it.", e1);

        //there is a known bug in OpenJDK that throws java.lang.ClassNotFoundException: com.sun.deploy.uitoolkit.impl.fx.HostServicesFactory
        //See https://bugs.openjdk.java.net/browse/JDK-8160464   (seems like fix is only for java 9)

        //this code is part of that fix:   (I first tried  Desktop.getDesktop().browse(new URI(url)); but that didn't work)

        try { String osName = System.getProperty("os.name");
            if (osName.startsWith("Mac OS")) {
                Class.forName("com.apple.eio.FileManager").getDeclaredMethod(
                        "openURL", new Class[]{String.class}).invoke(null,
                        new Object[]{url});
            } else if (osName.startsWith("Windows")) {
                Runtime.getRuntime().exec(
                        "rundll32 url.dll,FileProtocolHandler " + url);
            } else { //assume Unix or Linux
                String browser = null;
                final String[] browsers = {"google-chrome", "firefox", "opera", "konqueror", "mozilla"};
                for (String b : browsers) {
                    if (browser == null && Runtime.getRuntime().exec(
                            new String[]{"which", b}).getInputStream().read() != -1) {
                        Runtime.getRuntime().exec(new String[]{browser = b, url});
                    }
                }

                if (browser == null) {
                    throw new Exception("No web browser found");
                }
            }
        } catch (Exception e2) {
            LOG.error("Error while trying to open external browser with fallback method. Will throw uncaught exception now.", e2);
            throw new RuntimeException("Failed to open external browser (used 2 methods)", e2);
        }
    }
        return true;
    }

}
